###################################################
### chunk number 1: 
###################################################
options(width=65)


###################################################
### chunk number 2: 
###################################################
rm(list = ls())
setwd("C:\\temp\\course materials\\Day 2\\No. 1\\")
stage <- read.csv("stage.csv")
stage$Tree.ID <- factor(stage$Tree.ID)
stage$Forest.ID <- factor(stage$Forest, labels = c("Kaniksu", 
    "Coeur d'Alene", "St. Joe", "Clearwater", "Nez Perce", "Clark Fork", 
    "Umatilla", "Wallowa", "Payette"))
stage$HabType.ID <- factor(stage$HabType, labels = c("Ts/Pach", 
    "Ts/Op", "Th/Pach", "AG/Pach", "PA/Pach"))
stage$dbhib.cm <- stage$Dbhib * 2.54
stage$height.m <- stage$Height / 3.2808399


###################################################
### chunk number 3: fig-stage-1
###################################################
opar <- par(las=1)
plot(stage$dbhib.cm, stage$height.m, xlab="Dbhib (cm)", ylab="Height (m)")
par(opar)


###################################################
### chunk number 4: stage1
###################################################
opar <- par(las=1)
plot(stage$dbhib.cm, stage$height.m, xlab="Dbhib (cm)", ylab="Height (m)")
par(opar)


###################################################
### chunk number 5: fig-stage-2-old
###################################################
colours <- c("deepskyblue","goldenrod","purple",
             "orangered2","seagreen")
par(mfrow=c(3,3), pty="m", mar=c(2, 2, 3, 1) + 0.1, las=1)
for (i in 1:length(levels(stage$Forest.ID))) {
  thisForest <- levels(stage$Forest.ID)[i]
  forestData <- stage[stage$Forest.ID==thisForest,]
  plot(stage$dbhib.cm, stage$height.m, xlab = "", ylab = "",
       main = thisForest, type="n")
  theseTrees <- factor(forestData$Tree.ID)
  legend("topleft",
         unique(as.character(forestData$HabType.ID)), 
         xjust=0, yjust=1, bty="n",
         col=colours[unique(forestData$HabType)],
         lty=unique(forestData$HabType)+1)
  for (j in 1:length(levels(theseTrees))) {
    thisTree <- levels(theseTrees)[j]
    lines(forestData$dbhib.cm[forestData$Tree.ID==thisTree],
          forestData$height.m[forestData$Tree.ID==thisTree],
          col=colours[forestData$HabType[forestData$Tree.ID==thisTree]],
          lty=forestData$HabType[forestData$Tree.ID==thisTree]+1)
  }
}
mtext("Height (m)", outer=T, side=2, line=2)
mtext("Diameter (cm)", outer=T, side=1, line=2)


###################################################
### chunk number 6: 
# you will need to add ggplot2.  Go to packages
# then Set Cran Mirror, and then Install packages
# and find ggplot2 in the list.
###################################################
require(ggplot2)


###################################################
### chunk number 7: fig-stage-2
###################################################
qplot(dbhib.cm, height.m, 
      data = stage, group = Tree.ID,
      geom = "line", 
      facets = ~ Forest.ID,
      colour = HabType.ID, 
      linetype = HabType.ID, 
      xlab = "Diameter (cm)", ylab = "Height (m)"
      ) +
  scale_colour_manual(name = "Habitat Type", 
                      values = c("deepskyblue","goldenrod","purple",
                        "orangered2","seagreen")) + 
  scale_linetype_manual(name = "Habitat Type", values = c(1,2,4:6))


###################################################
### chunk number 8: stage2
###################################################
print(
qplot(dbhib.cm, height.m, 
      data = stage, group = Tree.ID,
      geom = "line", 
      facets = ~ Forest.ID,
      colour = HabType.ID, 
      linetype = HabType.ID, 
      xlab = "Diameter (cm)", ylab = "Height (m)"
      ) +
  scale_colour_manual(name = "Habitat Type", 
                      values = c("deepskyblue","goldenrod","purple",
                        "orangered2","seagreen")) + 
  scale_linetype_manual(name = "Habitat Type", values = c(1,2,4:6))
      )


###################################################
### chunk number 9: sampletrees
###################################################
trees <- data.frame(plot=factor(c(1, 1, 1, 2, 2, 2)), 
                    dbh.cm=c(30, 32, 35, 30, 33, 35), 
                    ht.m=c(25, 30, 40, 30, 40, 50))


###################################################
### chunk number 10: fig-cs-1
###################################################
plot(trees$dbh.cm, trees$ht.m, pch=c(1, 19)[trees$plot], xlim=c(29, 36),
     xlab="Diameter (cm)", ylab="Height (m)")
abline(lm(ht.m ~ dbh.cm, data=trees), col="darkgrey")


###################################################
### chunk number 11: fig-cs-2
###################################################
case.model.1 <- lm(ht.m ~ dbh.cm, data=trees) 
plot(fitted(case.model.1), residuals(case.model.1), ylab="Residuals", 
            xlab="Fitted Values", pch=c(1, 19)[trees$plot])
abline(h=0, col="darkgrey")


###################################################
### chunk number 12: fig-cs-3
###################################################
case.model.2 <- lm(ht.m ~ dbh.cm*plot, data=trees) 
plot(fitted(case.model.2), residuals(case.model.2), 
     xlab="Fitted Values", ylab="Residuals", 
     pch=c(1, 19)[trees$plot])
abline(h=0, col="darkgrey")


###################################################
### chunk number 13: case-data
###################################################
plot(trees$dbh.cm, trees$ht.m, pch=c(1, 19)[trees$plot], xlim=c(29, 36),
     xlab="Diameter (cm)", ylab="Height (m)")
abline(lm(ht.m ~ dbh.cm, data=trees), col="darkgrey")


###################################################
### chunk number 14: case-res-2
###################################################
case.model.1 <- lm(ht.m ~ dbh.cm, data=trees) 
plot(fitted(case.model.1), residuals(case.model.1), ylab="Residuals", 
            xlab="Fitted Values", pch=c(1, 19)[trees$plot])
abline(h=0, col="darkgrey")


###################################################
### chunk number 15: case-res-3
###################################################
case.model.2 <- lm(ht.m ~ dbh.cm*plot, data=trees) 
plot(fitted(case.model.2), residuals(case.model.2), 
     xlab="Fitted Values", ylab="Residuals", 
     pch=c(1, 19)[trees$plot])
abline(h=0, col="darkgrey")


###################################################
### chunk number 16: 
###################################################
require(nlme)
require(lattice)


###################################################
### chunk number 17: 
###################################################
straw <- data.frame(y = c(10.1, 14.9, 15.9, 13.1, 4.2, 4.8, 5.8, 1.2),
                    x = c(1, 2, 3, 4, 1, 2, 3, 4),
                    group = factor(c(1, 1, 1, 1, 2, 2, 2, 2)))


###################################################
### chunk number 18: fig-simple1
###################################################
colours = c("red", "blue")
plot(straw$x, straw$y, col = colours[straw$group])


###################################################
### chunk number 19: simple1
###################################################
colours = c("red", "blue")
plot(straw$x, straw$y, col = colours[straw$group])


###################################################
### chunk number 20: 
###################################################
basic.1 <- lm(y ~ x, data=straw)


###################################################
### chunk number 21: 
###################################################
basic.2 <- lm(y ~ x + group, data=straw)


###################################################
### chunk number 22: 
###################################################
basic.3 <- lm(y ~ x * group, data=straw)


###################################################
### chunk number 23: 
###################################################
straw.mixed <- groupedData(y ~ x | group, data=straw)


###################################################
### chunk number 24: 
###################################################
basic.4 <- lme(y ~ x, random = ~1 | group, data = straw.mixed)


###################################################
### chunk number 25: fig-simple4
###################################################
plot(augPred(basic.4))


###################################################
### chunk number 26: capture4
###################################################
summary(basic.4)


###################################################
### chunk number 27: capture4
###################################################
summ.4 <- capture.output(summary(basic.4))


###################################################
### chunk number 28: capture4a
###################################################
cat(summ.4[1:4], sep="\n")


###################################################
### chunk number 29: capture4b
###################################################
cat(summ.4[6:9], sep="\n")


###################################################
### chunk number 30: capture4c
###################################################
cat(summ.4[11:17], sep="\n")


###################################################
### chunk number 31: capture4e
###################################################
cat(summ.4[19:21], sep="\n")


###################################################
### chunk number 32: capture4f
###################################################
cat(summ.4[23:24], sep="\n")


###################################################
### chunk number 33: 
###################################################
basic.5 <- lme(y ~ x, random = ~1 | group, 
               weights = varIdent(form = ~1 | group), 
               data = straw.mixed)


###################################################
### chunk number 34: capture5
###################################################
summary(basic.5)


###################################################
### chunk number 35: capture5
###################################################
summ.5 <- capture.output(summary(basic.5))


###################################################
### chunk number 36: capture5f
###################################################
cat(summ.5[11:16], sep="\n")


###################################################
### chunk number 37: 
###################################################
basic.6 <- lme(y ~ x, random = ~1 | group, 
               weights = varIdent(form = ~1 | group), 
               correlation = corAR1(), 
               data = straw.mixed)


###################################################
### chunk number 38: capture6
###################################################
summary(basic.6)


###################################################
### chunk number 39: capture6
###################################################
summ.6 <- capture.output(summary(basic.6))


###################################################
### chunk number 40: capture6f
###################################################
cat(summ.6[11:15], sep="\n")


###################################################
### chunk number 41: fig-simple7
###################################################
opar <- par(las=1)
colours <- c("blue", "darkgreen", "plum")
plot(straw$x, straw$y)
for (g in 1:2) lines(straw$x[straw$group == levels(straw$group)[g]], 
                     fitted(basic.1)[straw$group == 
                                     levels(straw$group)[g]], 
                     col = colours[1])
for (g in 1:2) lines(straw$x[straw$group == levels(straw$group)[g]], 
                     fitted(basic.2)[straw$group == 
                                     levels(straw$group)[g]], 
                     col = colours[2])
for (g in 1:2) lines(straw$x[straw$group == levels(straw$group)[g]], 
                     fitted(basic.4)[straw$group == 
                                     levels(straw$group)[g]], 
                     col = colours[3])
legend(2.5, 13, lty = rep(1, 3), col = colours, 
       legend = c("Mean Only", "Intercept Fixed", "Intercept Random"))
par(opar)


###################################################
### chunk number 42: simple4
###################################################
print(
plot(augPred(basic.4))
)


###################################################
### chunk number 43: simple7
###################################################
opar <- par(las=1)
colours <- c("blue", "darkgreen", "plum")
plot(straw$x, straw$y)
for (g in 1:2) lines(straw$x[straw$group == levels(straw$group)[g]], 
                     fitted(basic.1)[straw$group == 
                                     levels(straw$group)[g]], 
                     col = colours[1])
for (g in 1:2) lines(straw$x[straw$group == levels(straw$group)[g]], 
                     fitted(basic.2)[straw$group == 
                                     levels(straw$group)[g]], 
                     col = colours[2])
for (g in 1:2) lines(straw$x[straw$group == levels(straw$group)[g]], 
                     fitted(basic.4)[straw$group == 
                                     levels(straw$group)[g]], 
                     col = colours[3])
legend(2.5, 13, lty = rep(1, 3), col = colours, 
       legend = c("Mean Only", "Intercept Fixed", "Intercept Random"))
par(opar)


###################################################
### chunk number 44: 
###################################################
rm(list = ls())
stage <- read.csv("stage.csv")
dim(stage)
names(stage)
sapply(stage, class)


###################################################
### chunk number 45: 
###################################################
stage$Tree.ID <- factor(stage$Tree.ID)
stage$Forest.ID <- factor(stage$Forest, labels = c("Kan", 
    "Cd'A", "StJoe", "Cw", "NP", "CF", 
    "Uma", "Wall", "Ptte"))
stage$HabType.ID <- factor(stage$HabType, labels = c("Ts/Pach", 
    "Ts/Op", "Th/Pach", "AG/Pach", "PA/Pach"))


###################################################
### chunk number 46: 
###################################################
stage$dbhib.cm <- stage$Dbhib * 2.54
stage$height.m <- stage$Height / 3.2808399
str(stage)


###################################################
### chunk number 47: 
###################################################
stage.old <- stage[stage$Decade == 0, ]


###################################################
### chunk number 48: 
###################################################
hd.lm.1 <- lm(height.m ~ dbhib.cm * HabType.ID, 
        data = stage.old, 
        subset = HabType.ID !=  "Ts/Op")


###################################################
### chunk number 49: fig-hd-lm-1
###################################################
opar <- par(mfrow=c(2,2), mar=c(4, 4, 4, 1))
plot(hd.lm.1)
par(opar)


###################################################
### chunk number 50: s-hd-lm-1
###################################################
opar <- par(mfrow=c(2,2), mar=c(4, 4, 4, 1))
plot(hd.lm.1)
par(opar)


###################################################
### chunk number 51: 
###################################################
summary(hd.lm.1)


###################################################
### chunk number 52: 
###################################################
sd(stage.old$height.m)
summary(hd.lm.1)$sigma


###################################################
### chunk number 53: 
###################################################
summary(lm(height.m ~ dbhib.cm, data = stage.old, 
        subset = HabType.ID !=  "Ts/Op"))$sigma


###################################################
### chunk number 54: 
###################################################
require(nlme)
stage.old <- groupedData(height.m ~ dbhib.cm | Forest.ID, 
                         data = stage.old)


###################################################
### chunk number 55: 
###################################################
hd.lme.1 <- lme(height.m ~ dbhib.cm, random = ~1 | Forest.ID, 
      data = stage.old)


###################################################
### chunk number 56: fig-hd-lme-1
###################################################
opar <- par(mfrow = c(3, 2), mar = c(4, 4, 3, 1), las = 1, cex.axis = 0.9)
#### Plot 1
plot(fitted(hd.lme.1, level=0), stage.old$height.m, 
     xlab = "Fitted Values (height, m.)", 
     ylab = "Observed Values (height, m.)",  
     main = "Model Structure (I)")
abline(0, 1, col = "blue")
#### Plot 2
scatter.smooth(fitted(hd.lme.1), residuals(hd.lme.1, type="pearson"), 
               xlab = "Fitted Values", 
               ylab = "Innermost Residuals", 
               main = "Model Structure (II)")
abline(h = 0, col = "red")
#### Plot 3
ref.forest <- ranef(hd.lme.1)[[1]]
ref.var.forest <- tapply(residuals(hd.lme.1, type="pearson", level=1),
                      stage.old$Forest.ID,  var)
qqnorm(ref.forest, main="Q-Q Normal - Forest Random Effects")
qqline(ref.forest, col="red")
#### Plot 4
qqnorm(residuals(hd.lme.1, type="pearson"), main="Q-Q Normal - Residuals")
qqline(residuals(hd.lme.1, type="pearson"), col="red")
#### Plot 5
boxplot(residuals(hd.lme.1, type="pearson", level=1) ~ stage.old$Forest.ID, 
        ylab = "Innermost Residuals", xlab = "National Forest",
        notch=T, varwidth = T, at=rank(ref.forest))
axis(3, labels=format(ref.forest, dig=2), cex.axis=0.8,
     at=rank(ref.forest))
abline(h=0, col="darkgreen")
#### Plot 6
plot(ref.forest, ref.var.forest, xlab="Forest Random Effect",
     ylab="Variance of within-Forest Residuals")
abline(lm(ref.var.forest ~ ref.forest), col="purple")
par(opar) 


###################################################
### chunk number 57: diag-lme-1
###################################################
opar <- par(mfrow = c(3, 2), mar = c(4, 4, 3, 1), las = 1, cex.axis = 0.9)
#### Plot 1
plot(fitted(hd.lme.1, level=0), stage.old$height.m, 
     xlab = "Fitted Values (height, m.)", 
     ylab = "Observed Values (height, m.)",  
     main = "Model Structure (I)")
abline(0, 1, col = "blue")
#### Plot 2
scatter.smooth(fitted(hd.lme.1), residuals(hd.lme.1, type="pearson"), 
               xlab = "Fitted Values", 
               ylab = "Innermost Residuals", 
               main = "Model Structure (II)")
abline(h = 0, col = "red")
#### Plot 3
ref.forest <- ranef(hd.lme.1)[[1]]
ref.var.forest <- tapply(residuals(hd.lme.1, type="pearson", level=1),
                      stage.old$Forest.ID,  var)
qqnorm(ref.forest, main="Q-Q Normal - Forest Random Effects")
qqline(ref.forest, col="red")
#### Plot 4
qqnorm(residuals(hd.lme.1, type="pearson"), main="Q-Q Normal - Residuals")
qqline(residuals(hd.lme.1, type="pearson"), col="red")
#### Plot 5
boxplot(residuals(hd.lme.1, type="pearson", level=1) ~ stage.old$Forest.ID, 
        ylab = "Innermost Residuals", xlab = "National Forest",
        notch=T, varwidth = T, at=rank(ref.forest))
axis(3, labels=format(ref.forest, dig=2), cex.axis=0.8,
     at=rank(ref.forest))
abline(h=0, col="darkgreen")
#### Plot 6
plot(ref.forest, ref.var.forest, xlab="Forest Random Effect",
     ylab="Variance of within-Forest Residuals")
abline(lm(ref.var.forest ~ ref.forest), col="purple")
par(opar) 


###################################################
### chunk number 58: 
###################################################
summary(hd.lme.1)


###################################################
### chunk number 59: 
###################################################
anova(hd.lme.1)


###################################################
### chunk number 60: 
###################################################
require(nlme)
stage <- groupedData(height.m ~ dbhib.cm | Forest.ID/Tree.ID, data = stage)


###################################################
### chunk number 61: 
###################################################
hd.lme.3 <- lme(height.m ~ dbhib.cm, 
                random = ~1 | Forest.ID/Tree.ID, 
                data = stage)


###################################################
### chunk number 62: hd-lme-3a
###################################################
opar <- par(mfrow = c(1, 3), mar = c(4, 4, 3, 1), las = 1, 
        cex.axis = 0.9)
  plot(fitted(hd.lme.3, level=0), stage$height.m, 
      xlab = "Fitted Values", ylab = "Observed Values", 
      main = "Model Structure (I)")
  abline(0, 1, col = "gray")
  scatter.smooth(fitted(hd.lme.3), residuals(hd.lme.3, type="pearson"), 
      main = "Model Structure (II)", 
      xlab = "Fitted Values", ylab = "Innermost Residuals")
  abline(h = 0, col = "gray")
acf.resid <- ACF(hd.lme.3, resType = "normal")
plot(acf.resid$lag[acf.resid$lag < 10.5], 
     acf.resid$ACF[acf.resid$lag < 10.5], 
     type="b", main="Autocorrelation", 
     xlab="Lag", ylab="Correlation")
stdv <- qnorm(1 - 0.01/2)/sqrt(attr(acf.resid, "n.used"))
lines(acf.resid$lag[acf.resid$lag < 10.5], 
      stdv[acf.resid$lag < 10.5], 
      col="darkgray")
lines(acf.resid$lag[acf.resid$lag < 10.5], 
      -stdv[acf.resid$lag < 10.5], 
      col="darkgray")
abline(0,0,col="gray")
par(opar)


###################################################
### chunk number 63: hd-lme-3a-d
###################################################
opar <- par(mfrow = c(1, 3), mar = c(4, 4, 3, 1), las = 1, 
        cex.axis = 0.9)
  plot(fitted(hd.lme.3, level=0), stage$height.m, 
      xlab = "Fitted Values", ylab = "Observed Values", 
      main = "Model Structure (I)")
  abline(0, 1, col = "gray")
  scatter.smooth(fitted(hd.lme.3), residuals(hd.lme.3, type="pearson"), 
      main = "Model Structure (II)", 
      xlab = "Fitted Values", ylab = "Innermost Residuals")
  abline(h = 0, col = "gray")
acf.resid <- ACF(hd.lme.3, resType = "normal")
plot(acf.resid$lag[acf.resid$lag < 10.5], 
     acf.resid$ACF[acf.resid$lag < 10.5], 
     type="b", main="Autocorrelation", 
     xlab="Lag", ylab="Correlation")
stdv <- qnorm(1 - 0.01/2)/sqrt(attr(acf.resid, "n.used"))
lines(acf.resid$lag[acf.resid$lag < 10.5], 
      stdv[acf.resid$lag < 10.5], 
      col="darkgray")
lines(acf.resid$lag[acf.resid$lag < 10.5], 
      -stdv[acf.resid$lag < 10.5], 
      col="darkgray")
abline(0,0,col="gray")
par(opar)


###################################################
### chunk number 64: hd-lme-3b
###################################################
opar <- par(mfrow = c(1, 3), mar = c(4, 4, 3, 1), las = 1, 
        cex.axis = 0.9)
  ref.forest <- ranef(hd.lme.3, level=1, standard=T)[[1]]
  ref.tree <- ranef(hd.lme.3, level=2, standard=T)[[1]]

  ref.tree.frame <- ranef(hd.lme.3, level=2, augFrame=T, standard=T)

  ref.var.tree <- tapply(residuals(hd.lme.3, type="pearson", level=1),
                      stage$Tree.ID,  var)
  ref.var.forest <- tapply(ref.tree, ref.tree.frame$Forest, var)  

  qqnorm(ref.forest, main = "QQ plot: Forest")
  qqline(ref.forest)
  qqnorm(ref.tree, main = "QQ plot: Tree")
  qqline(ref.tree)
  qqnorm(residuals(hd.lme.3, type="pearson"), main="QQ plot: Residuals")
  qqline(residuals(hd.lme.3, type="pearson"), col="red")
par(opar)


###################################################
### chunk number 65: hd-lme-3b-d
###################################################
opar <- par(mfrow = c(1, 3), mar = c(4, 4, 3, 1), las = 1, 
        cex.axis = 0.9)
  ref.forest <- ranef(hd.lme.3, level=1, standard=T)[[1]]
  ref.tree <- ranef(hd.lme.3, level=2, standard=T)[[1]]

  ref.tree.frame <- ranef(hd.lme.3, level=2, augFrame=T, standard=T)

  ref.var.tree <- tapply(residuals(hd.lme.3, type="pearson", level=1),
                      stage$Tree.ID,  var)
  ref.var.forest <- tapply(ref.tree, ref.tree.frame$Forest, var)  

  qqnorm(ref.forest, main = "QQ plot: Forest")
  qqline(ref.forest)
  qqnorm(ref.tree, main = "QQ plot: Tree")
  qqline(ref.tree)
  qqnorm(residuals(hd.lme.3, type="pearson"), main="QQ plot: Residuals")
  qqline(residuals(hd.lme.3, type="pearson"), col="red")
par(opar)


###################################################
### chunk number 66: hd-lme-3c
###################################################
opar <- par(mfrow = c(2, 2), mar = c(4, 4, 3, 1), las = 1, 
        cex.axis = 0.9)

boxplot(ref.tree ~ ref.tree.frame$Forest, 
        ylab = "Tree Effects", xlab = "National Forest",
        notch=T, varwidth = T, at=rank(ref.forest))
axis(3, labels=format(ref.forest, dig=2), cex.axis=0.8,
     at=rank(ref.forest))
abline(h=0, col="darkgreen")

boxplot(residuals(hd.lme.3, type="pearson", level=1) ~ stage$Tree.ID, 
        ylab = "Innermost Residuals", xlab = "Tree",
        notch=T, varwidth = T, at=rank(ref.tree))
axis(3, labels=format(ref.tree, dig=2), cex.axis=0.8,
     at=rank(ref.tree))
abline(h=0, col="darkgreen")

plot(ref.forest, ref.var.forest, xlab="Forest Random Effect",
     ylab="Variance of within-Forest Residuals")
abline(lm(ref.var.forest ~ ref.forest), col="purple")

plot(ref.tree, ref.var.tree, xlab="Tree Random Effect",
     ylab="Variance of within-Tree Residuals")
abline(lm(ref.var.forest ~ ref.forest), col="purple")

par(opar)


###################################################
### chunk number 67: hd-lme-3c-d
###################################################
opar <- par(mfrow = c(2, 2), mar = c(4, 4, 3, 1), las = 1, 
        cex.axis = 0.9)

boxplot(ref.tree ~ ref.tree.frame$Forest, 
        ylab = "Tree Effects", xlab = "National Forest",
        notch=T, varwidth = T, at=rank(ref.forest))
axis(3, labels=format(ref.forest, dig=2), cex.axis=0.8,
     at=rank(ref.forest))
abline(h=0, col="darkgreen")

boxplot(residuals(hd.lme.3, type="pearson", level=1) ~ stage$Tree.ID, 
        ylab = "Innermost Residuals", xlab = "Tree",
        notch=T, varwidth = T, at=rank(ref.tree))
axis(3, labels=format(ref.tree, dig=2), cex.axis=0.8,
     at=rank(ref.tree))
abline(h=0, col="darkgreen")

plot(ref.forest, ref.var.forest, xlab="Forest Random Effect",
     ylab="Variance of within-Forest Residuals")
abline(lm(ref.var.forest ~ ref.forest), col="purple")

plot(ref.tree, ref.var.tree, xlab="Tree Random Effect",
     ylab="Variance of within-Tree Residuals")
abline(lm(ref.var.forest ~ ref.forest), col="purple")

par(opar)


###################################################
### chunk number 68: 
###################################################
trees.in.forests <- aggregate(x=list(measures=stage$height.m), 
    by=list(tree=stage$Tree.ID, forest=stage$Forest.ID), FUN=length)
panel.order <- rank(as.numeric(as.character(trees.in.forests$tree)))


###################################################
### chunk number 69: hd.tree
###################################################
plot(augPred(hd.lme.3), index.cond=list(panel.order))


###################################################
### chunk number 70: hd-tree
###################################################
print(
plot(augPred(hd.lme.3), index.cond=list(panel.order))
)


###################################################
### chunk number 71: hd.lme.4
###################################################
hd.lme.4 <- lme(height.m ~ dbhib.cm + I(dbhib.cm^2), 
   random = ~ dbhib.cm + I(dbhib.cm^2) | Tree.ID, 
     data = stage)


###################################################
### chunk number 72: hd-lme-4
###################################################
opar <- par(mfrow = c(1, 3), mar = c(4, 4, 3, 1), las = 1, 
        cex.axis = 0.9)
  plot(fitted(hd.lme.4, level=0), stage$height.m, 
      xlab = "Fitted Values", ylab = "Observed Values", 
      main = "Model Structure (I)")
  abline(0, 1, col = "gray")
  scatter.smooth(fitted(hd.lme.4), residuals(hd.lme.4, type="pearson"), 
      main = "Model Structure (II)", 
      xlab = "Fitted Values", ylab = "Innermost Residuals")
  abline(0, 0, col = "gray")
acf.resid <- ACF(hd.lme.4, resType = "n")
plot(acf.resid$lag[acf.resid$lag < 10.5], 
     acf.resid$ACF[acf.resid$lag < 10.5], 
     type="b", main="Autocorrelation", 
     xlab="Lag", ylab="Correlation")
stdv <- qnorm(1 - 0.01/2)/sqrt(attr(acf.resid, "n.used"))
lines(acf.resid$lag[acf.resid$lag < 10.5], 
      stdv[acf.resid$lag < 10.5], 
      col="darkgray")
lines(acf.resid$lag[acf.resid$lag < 10.5], 
      -stdv[acf.resid$lag < 10.5], 
      col="darkgray")
abline(0,0,col="gray")
par(opar)


###################################################
### chunk number 73: hd-lme-4-d
###################################################
opar <- par(mfrow = c(1, 3), mar = c(4, 4, 3, 1), las = 1, 
        cex.axis = 0.9)
  plot(fitted(hd.lme.4, level=0), stage$height.m, 
      xlab = "Fitted Values", ylab = "Observed Values", 
      main = "Model Structure (I)")
  abline(0, 1, col = "gray")
  scatter.smooth(fitted(hd.lme.4), residuals(hd.lme.4, type="pearson"), 
      main = "Model Structure (II)", 
      xlab = "Fitted Values", ylab = "Innermost Residuals")
  abline(0, 0, col = "gray")
acf.resid <- ACF(hd.lme.4, resType = "n")
plot(acf.resid$lag[acf.resid$lag < 10.5], 
     acf.resid$ACF[acf.resid$lag < 10.5], 
     type="b", main="Autocorrelation", 
     xlab="Lag", ylab="Correlation")
stdv <- qnorm(1 - 0.01/2)/sqrt(attr(acf.resid, "n.used"))
lines(acf.resid$lag[acf.resid$lag < 10.5], 
      stdv[acf.resid$lag < 10.5], 
      col="darkgray")
lines(acf.resid$lag[acf.resid$lag < 10.5], 
      -stdv[acf.resid$lag < 10.5], 
      col="darkgray")
abline(0,0,col="gray")
par(opar)


###################################################
### chunk number 74: hd.lme.5
###################################################
hd.lme.5 <- update(hd.lme.4, correlation = corCAR1(),
                   control = lmeControl(opt="optim"))


###################################################
### chunk number 75: hd-lme-5
###################################################
opar <- par(mfrow = c(1, 3), mar = c(4, 4, 3, 1), las = 1, 
        cex.axis = 0.9)
  plot(fitted(hd.lme.5, level=0), stage$height.m, 
      xlab = "Fitted Values", ylab = "Observed Values", 
      main = "Model Structure (I)")
  abline(0, 1, col = "gray")
  scatter.smooth(fitted(hd.lme.5), residuals(hd.lme.5, type="pearson"), 
      main = "Model Structure (II)", 
      xlab = "Fitted Values", ylab = "Innermost Residuals")
  abline(0, 0, col = "gray")
acf.resid <- ACF(hd.lme.5, resType = "n")
plot(acf.resid$lag[acf.resid$lag < 10.5], 
     acf.resid$ACF[acf.resid$lag < 10.5], 
     type="b", main="Autocorrelation", 
     xlab="Lag", ylab="Correlation")
stdv <- qnorm(1 - 0.01/2)/sqrt(attr(acf.resid, "n.used"))
lines(acf.resid$lag[acf.resid$lag < 10.5], 
      stdv[acf.resid$lag < 10.5], 
      col="darkgray")
lines(acf.resid$lag[acf.resid$lag < 10.5], 
      -stdv[acf.resid$lag < 10.5], 
      col="darkgray")
abline(0,0,col="gray")
par(opar)


###################################################
### chunk number 76: hd-lme-5-d
###################################################
opar <- par(mfrow = c(1, 3), mar = c(4, 4, 3, 1), las = 1, 
        cex.axis = 0.9)
  plot(fitted(hd.lme.5, level=0), stage$height.m, 
      xlab = "Fitted Values", ylab = "Observed Values", 
      main = "Model Structure (I)")
  abline(0, 1, col = "gray")
  scatter.smooth(fitted(hd.lme.5), residuals(hd.lme.5, type="pearson"), 
      main = "Model Structure (II)", 
      xlab = "Fitted Values", ylab = "Innermost Residuals")
  abline(0, 0, col = "gray")
acf.resid <- ACF(hd.lme.5, resType = "n")
plot(acf.resid$lag[acf.resid$lag < 10.5], 
     acf.resid$ACF[acf.resid$lag < 10.5], 
     type="b", main="Autocorrelation", 
     xlab="Lag", ylab="Correlation")
stdv <- qnorm(1 - 0.01/2)/sqrt(attr(acf.resid, "n.used"))
lines(acf.resid$lag[acf.resid$lag < 10.5], 
      stdv[acf.resid$lag < 10.5], 
      col="darkgray")
lines(acf.resid$lag[acf.resid$lag < 10.5], 
      -stdv[acf.resid$lag < 10.5], 
      col="darkgray")
abline(0,0,col="gray")
par(opar)


###################################################
### chunk number 77: hab5a
###################################################
plot(hd.lme.5, resid(.) ~ fitted(.) | HabType.ID, layout=c(1, 5))


###################################################
### chunk number 78: hab5b
###################################################
qqmath(~ resid(hd.lme.5) | stage$HabType.ID,
       prepanel = prepanel.qqmathline,
       panel = function(x, ...) {
         panel.qqmathline(x, distribution = qnorm)
         panel.qqmath(x, ...)
       })


###################################################
### chunk number 79: habitat5a
###################################################
print(
plot(hd.lme.5, resid(.) ~ fitted(.) | HabType.ID, layout=c(1, 5))
      )


###################################################
### chunk number 80: habitat5b
###################################################
print(
qqmath(~ resid(hd.lme.5) | stage$HabType.ID,
       prepanel = prepanel.qqmathline,
       panel = function(x, ...) {
         panel.qqmathline(x, distribution = qnorm)
         panel.qqmath(x, ...)
       })
)


###################################################
### chunk number 81: hd.tree.5
###################################################
plot(augPred(hd.lme.5), index.cond=list(panel.order))


###################################################
### chunk number 82: hd-tree-5
###################################################
print(
plot(augPred(hd.lme.5), index.cond=list(panel.order))
  )


###################################################
### chunk number 83: 
###################################################
summary(hd.lme.5)


###################################################
### chunk number 84: addedv
###################################################
age.lme.1 <- lme(Age ~ dbhib.cm, random = ~1 | Forest.ID/Tree.ID, 
      data = stage)
res.Age <- residuals(age.lme.1, level = 0)
res.HD <- residuals(hd.lme.5, level = 0)
scatter.smooth(res.Age, res.HD, xlab = "Variation unique to Age", 
     ylab = "Variation in Height after all but Age")


###################################################
### chunk number 85: added
###################################################
age.lme.1 <- lme(Age ~ dbhib.cm, random = ~1 | Forest.ID/Tree.ID, 
      data = stage)
res.Age <- residuals(age.lme.1, level = 0)
res.HD <- residuals(hd.lme.5, level = 0)
scatter.smooth(res.Age, res.HD, xlab = "Variation unique to Age", 
     ylab = "Variation in Height after all but Age")


###################################################
### chunk number 86: addedhab
###################################################
xyplot(stage$height.m ~ fitted(hd.lme.5, level=0) | HabType.ID,
        xlab="Predicted height (m)",
        ylab="Observed height (m)",
        data=stage,
        panel = function(x, y, subscripts) {
                panel.xyplot(x, y)
                panel.abline(0, 1)
                panel.abline(lm(y ~ x), lty=3)
        }
)


###################################################
### chunk number 87: hd-hab
###################################################
print(
xyplot(stage$height.m ~ fitted(hd.lme.5, level=0) | HabType.ID,
        xlab="Predicted height (m)",
        ylab="Observed height (m)",
        data=stage,
        panel = function(x, y, subscripts) {
                panel.xyplot(x, y)
                panel.abline(0, 1)
                panel.abline(lm(y ~ x), lty=3)
        }
)
)


###################################################
### chunk number 88: hackknife
###################################################
all.betas <- data.frame(labels=names(unlist(hd.lme.1$coefficients)))
cook.0 <- cook.1 <- rep(NA, dim(stage.old)[1])

p.sigma.0 <- length(hd.lme.1$coefficients$fixed) *
         var(residuals(hd.lme.1, level=0))
p.sigma.1 <- length(hd.lme.1$coefficients$fixed) *
         var(residuals(hd.lme.1, level=1))

for (i in 1:dim(stage.old)[1]) {
  try({ hd.lme.n <- update(hd.lme.1, data = stage.old[-i,])
        new.betas <- data.frame(labels=names(unlist(hd.lme.n$coefficients)),
                                coef=unlist(hd.lme.n$coefficients))
        names(new.betas)[2] <- paste("obs", i, sep=".")
        all.betas <- merge(all.betas, new.betas, all.x = TRUE)
        cook.0[i] <- sum((predict(hd.lme.1, level=0, newdata=stage.old) -
                         predict(hd.lme.n, level=0, newdata=stage.old))^2) /
                           p.sigma.0
        cook.1[i] <- sum((predict(hd.lme.1, level=1, newdata=stage.old) -
                         predict(hd.lme.n, level=1, newdata=stage.old))^2) /
                           p.sigma.1
      })
}


###################################################
### chunk number 89: 
###################################################
all.betas <- t(all.betas[,-1])

len.all <- length(unlist(hd.lme.1$coefficients))
len.fixed <- length(hd.lme.1$coefficients$fixed) # 2
len.ran <- length(hd.lme.1$coefficients$random$Forest.ID) # 9


###################################################
### chunk number 90: fig-hd-lme-1-param
###################################################
opar <- par(mfrow=c(len.all, 1), oma=c(2,0,1,0), mar=c(0,4,0,0), las=1)
for (i in 1:len.fixed) {
  plot(all.betas[,i], type="l", axes=F, xlab="", ylab="")
  text(length(all.betas[,i])-1, max(all.betas[,i], na.rm=T),
       names(unlist(hd.lme.1$coefficients))[i],
       adj=c(1,1), col="red")
  axis(2)
  box()
}
for (i in (len.fixed+1):(len.all)) {
  plot(all.betas[,i], type="l", axes=F, xlab="", ylab="")
  text(length(all.betas[,i])-1, max(all.betas[,i], na.rm=T),
       names(unlist(hd.lme.1$coefficients))[i],
       adj=c(1,1), col="red")
  axis(2)
  box()
}
axis(1)
par(opar)


###################################################
### chunk number 91: diag-lme-1-p
###################################################
opar <- par(mfrow=c(len.all, 1), oma=c(2,0,1,0), mar=c(0,4,0,0), las=1)
for (i in 1:len.fixed) {
  plot(all.betas[,i], type="l", axes=F, xlab="", ylab="")
  text(length(all.betas[,i])-1, max(all.betas[,i], na.rm=T),
       names(unlist(hd.lme.1$coefficients))[i],
       adj=c(1,1), col="red")
  axis(2)
  box()
}
for (i in (len.fixed+1):(len.all)) {
  plot(all.betas[,i], type="l", axes=F, xlab="", ylab="")
  text(length(all.betas[,i])-1, max(all.betas[,i], na.rm=T),
       names(unlist(hd.lme.1$coefficients))[i],
       adj=c(1,1), col="red")
  axis(2)
  box()
}
axis(1)
par(opar)


###################################################
### chunk number 92: fig-hd-lme-1-cook
###################################################
cook <- data.frame(id=stage.old$Tree.ID, fixed=cook.0, forest=cook.1)
influential <- apply(cook[,2:3], 1, max) > 1
plot(cook$fixed, cook$forest, type="n",
    xlab="Outermost (Fixed effects only)", 
    ylab="Innermost (Fixed effects and random effects)")
points(cook$fixed[!influential], cook$forest[!influential])
if(sum(influential) > 0)
   text(cook$fixed[influential], cook$forest[influential], 
      cook$id[influential], col="red", cex=0.85)


###################################################
### chunk number 93: diag-lme-1-cook
###################################################
cook <- data.frame(id=stage.old$Tree.ID, fixed=cook.0, forest=cook.1)
influential <- apply(cook[,2:3], 1, max) > 1
plot(cook$fixed, cook$forest, type="n",
    xlab="Outermost (Fixed effects only)", 
    ylab="Innermost (Fixed effects and random effects)")
points(cook$fixed[!influential], cook$forest[!influential])
if(sum(influential) > 0)
   text(cook$fixed[influential], cook$forest[influential], 
      cook$id[influential], col="red", cex=0.85)


